import re
from django.db import connection
from reportserver.lib.io.requests import get_selected_options_from
from reportserver.lib.db.where import Where
from collections import namedtuple

Columns = namedtuple("Columns", "nuts1 regioncategory beneficiaryid gender")


def where_filter(request, table):
    """
    takes options from request and builds with according columns
    a where clause for SQL
    :param request:
    :param table:
    :return:
    """
    selected_options = get_selected_options_from(request)
    return build_where_filter(selected_options)


def build_where_filter(selected_options):
    """
    builder method for where filter
    takes columns and options and builds where-filter
    :param columns:
    :param selected_options:
    :return:
    """
    where = Where()
    columns = get_columns()

    # Start and end date within interval
    if selected_options.filter_type == "1":
        where.add_clause("start_date >= %s", [selected_options.start])
        where.add_clause("end_date IS NOT NULL AND end_date <= %s",
                         [selected_options.end])

    # Start date within interval
    elif selected_options.filter_type == "2":
        where.add_clause("start_date >= %s", [selected_options.start])
        where.add_clause("start_date <= %s", [selected_options.end])

    # End date within interval
    elif selected_options.filter_type == "3":
        where.add_clause("end_date IS NOT NULL", [])
        where.add_clause("end_date >= %s", [selected_options.start])
        where.add_clause("end_date <= %s", [selected_options.end])

    # Start or end date within interval
    elif selected_options.filter_type == "4":
        where.add_clause("((start_date >= %s AND start_date <= %s) OR "
                         "(end_date IS NOT NULL AND end_date >= %s AND "
                         " end_date <= %s))",
                         [selected_options.start, selected_options.end,
                          selected_options.start, selected_options.end])

    # Case active within interval
    elif selected_options.filter_type == "5":
        where.add_clause("(start_date <= %s) AND "
                         "(end_date IS NULL OR end_date >= %s)",
                         [selected_options.end, selected_options.start])

    if selected_options.nuts1:
        where.add_clause(columns.nuts1, [selected_options.nuts1])
    if selected_options.regioncategory:
        regioncategories = [int(v) for v in selected_options.regioncategory]
        where.add_clause(columns.regioncategory, [regioncategories])
    if selected_options.beneficiary:
        where.add_clause(columns.beneficiaryid, [selected_options.beneficiary])
    if selected_options.gender:
        gender = [int(v) for v in selected_options.gender]
        where.add_clause(columns.gender, [gender])

    if selected_options.foerderphase:
        foerderphase = [int(v) for v in selected_options.foerderphase]
        if 0 in foerderphase and 1 in foerderphase:
            pass
        elif 0 in foerderphase:
            where.add_clause("(start_date < '2019-01-01'::date)", [])
        elif 1 in foerderphase:
            where.add_clause("(start_date > '2018-12-31'::date)", [])
    return where


def get_columns():
    """
    columns for participation and virtual participations
    :return: Columns named tuple
    """
    return Columns("nuts1 = ANY(%s)",
                   "regioncategory_id = ANY(%s)",
                   "originating_beneficiary_id = ANY(%s)",
                   "gender = ANY(%s)")


def get_export_data(request, table):
    filter_ = where_filter(request, table)
    rows = get_rows(table, filter_)
    return rows

def enlarge_header(name):
    if name in ("cm", "cl", "ajs", "mp"):
        return name.upper()
    indicators_expression = r"c(o|r)\d{2}|b3_2|po04|po02|a2_1|c1_1|c1_2"
    if re.match(indicators_expression, name):
        return name.upper().replace("_", ".")
    return name


def generator_rows_with_header(cursor):
    """
    gives the result from DB back as generator
    :param cursor:
    :return:
    """
    yield [enlarge_header(col[0]) for col in cursor.description]
    for row in cursor.fetchall():
        yield row


def remove_id_column(rows):
    first_item_in_row_is_id = False
    first_row = rows.next()
    if first_row[0] == "id":
        first_item_in_row_is_id = True
    yield [item for k,item in enumerate(first_row)
           if k > 0 or not first_item_in_row_is_id]
    for row in rows:
        yield [item for k,item in enumerate(row)
           if k > 0 or not first_item_in_row_is_id]


def get_rows(table_name, where):
    """
    retrieves rows from DB
    :param table_name:
    :param where:
    :return:
    """
    cursor = connection.cursor()
    cursor.execute("select * from " + table_name + " where " + where.clause,
                   where.params)
    rows = generator_rows_with_header(cursor)
    rows = remove_id_column(rows)

    return rows
